# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.server.display import Display as ServerDisplay
from pywayland.client.display import Display as ClientDisplay

import pytest


def test_display_socket():
    """Test that we can create a socket and have a client connect to it"""
    s = ServerDisplay()

    s.add_socket("wayland-0")

    with pytest.raises(Exception):
        s.add_socket("wayland-0")

    c = ClientDisplay()

    c.connect()

    assert c._ptr

    c.disconnect()
    s.destroy()



# This file has been autogenerated by the pywayland scanner

# Copyright 2016 The Chromium Authors.
# Copyright 2017 Intel Corporation
# Copyright 2018 Collabora, Ltd
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpLinuxBufferReleaseV1(Interface):
    """Buffer release explicit synchronization

    This object is instantiated in response to a
    :func:`ZwpLinuxSurfaceSynchronizationV1.get_release()
    <pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxSurfaceSynchronizationV1.get_release>`
    request.

    It provides an alternative to :func:`WlBuffer.release()
    <pywayland.protocol.wayland.WlBuffer.release>` events, providing a unique
    release from a single :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` request. The release event
    also supports explicit synchronization, providing a fence FD for the client
    to synchronize against.

    Exactly one event, either a fenced_release or an immediate_release, will be
    emitted for the :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` request. The compositor can
    choose release by release which event it uses.

    This event does not replace :func:`WlBuffer.release()
    <pywayland.protocol.wayland.WlBuffer.release>` events; servers are still
    required to send those events.

    Once a buffer release object has delivered a 'fenced_release' or an
    'immediate_release' event it is automatically destroyed.
    """

    name = "zwp_linux_buffer_release_v1"
    version = 1


class ZwpLinuxBufferReleaseV1Proxy(Proxy):
    interface = ZwpLinuxBufferReleaseV1


class ZwpLinuxBufferReleaseV1Resource(Resource):
    interface = ZwpLinuxBufferReleaseV1

    @ZwpLinuxBufferReleaseV1.event(
        Argument(ArgumentType.FileDescriptor),
    )
    def fenced_release(self, fence):
        """Release buffer with fence

        Sent when the compositor has finalised its usage of the associated
        buffer for the relevant commit, providing a dma_fence which will be
        signaled when all operations by the compositor on that buffer for that
        commit have finished.

        Once the fence has signaled, and assuming the associated buffer is not
        pending release from other :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` requests, no additional
        explicit or implicit synchronization is required to safely reuse or
        destroy the buffer.

        This event destroys the :class:`ZwpLinuxBufferReleaseV1` object.

        :param fence:
            fence for last operation on buffer
        :type fence:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, fence)

    @ZwpLinuxBufferReleaseV1.event()
    def immediate_release(self):
        """Release buffer immediately

        Sent when the compositor has finalised its usage of the associated
        buffer for the relevant commit, and either performed no operations
        using it, or has a guarantee that all its operations on that buffer for
        that commit have finished.

        Once this event is received, and assuming the associated buffer is not
        pending release from other :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` requests, no additional
        explicit or implicit synchronization is required to safely reuse or
        destroy the buffer.

        This event destroys the :class:`ZwpLinuxBufferReleaseV1` object.
        """
        self._post_event(1)


class ZwpLinuxBufferReleaseV1Global(Global):
    interface = ZwpLinuxBufferReleaseV1


ZwpLinuxBufferReleaseV1._gen_c()
ZwpLinuxBufferReleaseV1.proxy_class = ZwpLinuxBufferReleaseV1Proxy
ZwpLinuxBufferReleaseV1.resource_class = ZwpLinuxBufferReleaseV1Resource
ZwpLinuxBufferReleaseV1.global_class = ZwpLinuxBufferReleaseV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Iterator
import xml.etree.ElementTree as ET

from .argument import Argument
from .description import Description
from .method import Method
from .printer import Printer


@dataclass(frozen=True)
class Event(Method):
    """Scanner for event objects (server-side method)

    Required attributes: `name`

    Optional attributes: `since`

    Child elements: `description` and `arg``
    """

    method_type = "event"

    @classmethod
    def parse(cls, element: ET.Element) -> "Event":
        name = cls.parse_attribute(element, "name")
        if name in ("global", "import"):
            name += "_"

        return cls(
            name=name,
            since=cls.parse_optional_attribute(element, "since"),
            description=cls.parse_optional_child(element, Description, "description"),
            arg=cls.parse_repeated_child(element, Argument, "arg"),
        )

    @property
    def method_args(self) -> Iterator[str]:
        """Generator of the arguments to the method

        All arguments to be sent to `._post_event` must be passed in
        """
        for arg in self.arg:
            yield arg.name

    def output_doc_params(self, printer: Printer) -> None:
        """Aguments documented as parameters

        All arguments are event parameters.
        """
        for arg in self.arg:
            arg.output_doc_param(printer)

    def output_body(self, printer: Printer, opcode: int) -> None:
        """Output the body of the event to the printer"""
        args = ", ".join([str(opcode)] + list(self.method_args))
        printer("self._post_event({})".format(args))



from typing import overload, Any, Callable, TypeVar

class CData:
    def __getitem__(self, idx: int): ...
    # required for wl_list_for_each
    @property
    def next(self) -> CData: ...

class CDataArray:
    def __setitem__(self, idx: int, elem: CData) -> None: ...

class DispatcherFuncT: ...
class ResourceDestroyFuncT: ...
class NotifyFuncT: ...

# built-in cdata types
class CharCData(CData): ...

# wayland cdata types
class ClientCData(CData): ...
class DisplayCData(CData): ...
class EventLoopCData(CData): ...
class InterfaceCData(CData): ...
class ListCData(CData): ...

class ListenerCData(CData):
    link: ListCData
    notify: NotifyFuncT

class QueueCData(CData): ...
class ResourceCData(CData): ...

# custom cdata types
class ListenerContainerCData(CData):
    handle: CData
    destroy_listener: ListenerCData

_FuncType = Callable[..., Any]
_F = TypeVar("_F", bound=_FuncType)
_CDataT = TypeVar("_CDataT", bound=CData)

NULL: CData

@overload
def new(cdecl: str) -> CData: ...
@overload
def new(cdecl: str, init: Any) -> CDataArray: ...
def gc(
    cdata: _CDataT, destructor: None | Callable[[_CDataT], None], size: int = 0
) -> _CDataT: ...
def string(cdata: CharCData) -> str: ...
def release(cdata: CData) -> None: ...
def def_extern() -> Callable[[_F], _F]: ...
def new_handle(self: Any) -> CData: ...
def from_handle(cdata: CData) -> Any: ...
def cast(new_type: "str", cdata: CData, /) -> CData: ...
def addressof(cdata: _CDataT) -> _CDataT: ...



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import xml.etree.ElementTree as ET
import os
from dataclasses import dataclass
from typing import Dict, List, Optional

from .element import Element
from .copyright import Copyright, copyright_default
from .description import Description
from .interface import Interface
from .printer import Printer


@dataclass(frozen=True)
class Protocol(Element):
    """Protocol scanner object

    Main scanner object that acts on the input xml files to generate protocol
    files.

    Required attributes: `name`

    Child elements: `copyright?`, `description?`, and `interface+`

    :param input_file: Name of input XML file
    """

    name: str
    copyright: Optional[Copyright]
    description: Optional[Description]
    interface: List[Interface]

    @classmethod
    def parse_file(cls, input_file: str) -> "Protocol":
        if not os.path.exists(input_file):
            raise ValueError("Input xml file does not exist: {}".format(input_file))
        xmlroot = ET.parse(input_file).getroot()
        if xmlroot.tag != "protocol":
            raise ValueError(
                "Input file not a valid Wayland protocol file: {}".format(input_file)
            )

        return cls.parse(xmlroot)

    @classmethod
    def parse(cls, element: ET.Element) -> "Protocol":
        return cls(
            name=cls.parse_attribute(element, "name"),
            copyright=cls.parse_optional_child(element, Copyright, "copyright"),
            description=cls.parse_optional_child(element, Description, "description"),
            interface=cls.parse_repeated_child(element, Interface, "interface"),
        )

    def __repr__(self) -> str:
        return "Protocol({})".format(self.name)

    def output(self, output_dir: str, module_imports: Dict[str, str]) -> None:
        """Output the scanned files to the given directory

        :param output_dir: Path of directory to output protocol files to
        :type output_dir: string
        """
        protocol_name = self.name.replace("-", "_")

        output_dir = os.path.join(output_dir, protocol_name)
        if not os.path.exists(output_dir):
            os.mkdir(output_dir)

        # First, we'll create the __init__.py file
        printer = Printer(protocol_name)
        if self.copyright:
            self.copyright.output(printer)
        else:
            printer(copyright_default)

        printer()
        for iface in sorted(self.interface, key=lambda x: x.name):
            printer(
                "from .{} import {}  # noqa: F401".format(iface.name, iface.class_name)
            )

        init_path = os.path.join(output_dir, "__init__.py")
        with open(init_path, "wb") as f:
            printer.write(f)

        # Now build all the modules
        for iface in self.interface:
            module_path = os.path.join(output_dir, iface.name + ".py")

            printer = Printer(self.name.replace("-", "_"), iface.name, module_imports)
            if self.copyright:
                self.copyright.output(printer)
            else:
                printer(copyright_default)
            printer()

            iface.output(printer, module_imports)

            with open(module_path, "wb") as f:
                printer.write(f)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from .xdg_positioner import XdgPositioner


class XdgPopup(Interface):
    """Short-lived, popup surfaces for menus

    A popup surface is a short-lived, temporary surface. It can be used to
    implement for example menus, popovers, tooltips and other similar user
    interface concepts.

    A popup can be made to take an explicit grab. See :func:`XdgPopup.grab()`
    for details.

    When the popup is dismissed, a popup_done event will be sent out, and at
    the same time the surface will be unmapped. See the
    :func:`XdgPopup.popup_done()` event for details.

    Explicitly destroying the :class:`XdgPopup` object will also dismiss the
    popup and unmap the surface. Clients that want to dismiss the popup when
    another surface of their own is clicked should dismiss the popup using the
    destroy request.

    A newly created :class:`XdgPopup` will be stacked on top of all previously
    created :class:`XdgPopup` surfaces associated with the same
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.

    The parent of an :class:`XdgPopup` must be mapped (see the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` description) before the
    :class:`XdgPopup` itself.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the :class:`XdgPopup`
    state to take effect.
    """

    name = "xdg_popup"
    version = 3

    class error(enum.IntEnum):
        invalid_grab = 0


class XdgPopupProxy(Proxy):
    interface = XdgPopup

    @XdgPopup.request()
    def destroy(self):
        """Remove :class:`XdgPopup` interface

        This destroys the popup. Explicitly destroying the :class:`XdgPopup`
        object will also dismiss the popup, and unmap the surface.

        If this :class:`XdgPopup` is not the "topmost" popup, a protocol error
        will be sent.
        """
        self._marshal(0)
        self._destroy()

    @XdgPopup.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def grab(self, seat, serial):
        """Make the popup take an explicit grab

        This request makes the created popup take an explicit grab. An explicit
        grab will be dismissed when the user dismisses the popup, or when the
        client destroys the :class:`XdgPopup`. This can be done by the user
        clicking outside the surface, using the keyboard, or even locking the
        screen through closing the lid or a timeout.

        If the compositor denies the grab, the popup will be immediately
        dismissed.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The serial number of
        the event should be passed as 'serial'.

        The parent of a grabbing popup must either be an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface or another
        :class:`XdgPopup` with an explicit grab. If the parent is another
        :class:`XdgPopup` it means that the popups are nested, with this popup
        now being the topmost popup.

        Nested popups must be destroyed in the reverse order they were created
        in, e.g. the only popup you are allowed to destroy at all times is the
        topmost one.

        When compositors choose to dismiss a popup, they may dismiss every
        nested grabbing popup as well. When a compositor dismisses popups, it
        will follow the same dismissing order as required from the client.

        The parent of a grabbing popup must either be another :class:`XdgPopup`
        with an active explicit grab, or an :class:`XdgPopup` or
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel`, if there are no
        explicit grabs already taken.

        If the topmost grabbing popup is destroyed, the grab will be returned
        to the parent of the popup, if that parent previously had an explicit
        grab.

        If the parent is a grabbing popup which has already been dismissed,
        this popup will be immediately dismissed. If the parent is a popup that
        did not take an explicit grab, an error will be raised.

        During a popup grab, the client owning the grab will receive pointer
        and touch events for all their surfaces as normal (similar to an
        "owner-events" grab in X11 parlance), while the top most grabbing popup
        will always have keyboard focus.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(1, seat, serial)

    @XdgPopup.request(
        Argument(ArgumentType.Object, interface=XdgPositioner),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def reposition(self, positioner, token):
        """Recalculate the popup's location

        Reposition an already-mapped popup. The popup will be placed given the
        details in the passed
        :class:`~pywayland.protocol.xdg_shell.XdgPositioner` object, and a
        :func:`XdgPopup.repositioned()` followed by
        :func:`XdgPopup.configure()` and :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` will be emitted in
        response. Any parameters set by the previous positioner will be
        discarded.

        The passed token will be sent in the corresponding
        :func:`XdgPopup.repositioned()` event. The new popup position will not
        take effect until the corresponding configure event is acknowledged by
        the client. See :func:`XdgPopup.repositioned()` for details. The token
        itself is opaque, and has no other special meaning.

        If multiple reposition requests are sent, the compositor may skip all
        but the last one.

        If the popup is repositioned in response to a configure event for its
        parent, the client should send an
        :func:`XdgPositioner.set_parent_configure()
        <pywayland.protocol.xdg_shell.XdgPositioner.set_parent_configure>` and
        possibly an :func:`XdgPositioner.set_parent_size()
        <pywayland.protocol.xdg_shell.XdgPositioner.set_parent_size>` request
        to allow the compositor to properly constrain the popup.

        If the popup is repositioned together with a parent that is being
        resized, but not in response to a configure event, the client should
        send an :func:`XdgPositioner.set_parent_size()
        <pywayland.protocol.xdg_shell.XdgPositioner.set_parent_size>` request.

        :param positioner:
        :type positioner:
            :class:`~pywayland.protocol.xdg_shell.XdgPositioner`
        :param token:
            reposition request token
        :type token:
            `ArgumentType.Uint`
        """
        self._marshal(2, positioner, token)


class XdgPopupResource(Resource):
    interface = XdgPopup

    @XdgPopup.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def configure(self, x, y, width, height):
        """Configure the popup surface

        This event asks the popup surface to configure itself given the
        configuration. The configured state should not be applied immediately.
        See :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The x and y arguments represent the position the popup was placed at
        given the :class:`~pywayland.protocol.xdg_shell.XdgPositioner` rule,
        relative to the upper left corner of the window geometry of the parent
        surface.

        For version 2 or older, the configure event for an :class:`XdgPopup` is
        only ever sent once for the initial configuration. Starting with
        version 3, it may be sent again if the popup is setup with an
        :class:`~pywayland.protocol.xdg_shell.XdgPositioner` with set_reactive
        requested, or in response to :func:`XdgPopup.reposition()` requests.

        :param x:
            x position relative to parent surface window geometry
        :type x:
            `ArgumentType.Int`
        :param y:
            y position relative to parent surface window geometry
        :type y:
            `ArgumentType.Int`
        :param width:
            window geometry width
        :type width:
            `ArgumentType.Int`
        :param height:
            window geometry height
        :type height:
            `ArgumentType.Int`
        """
        self._post_event(0, x, y, width, height)

    @XdgPopup.event()
    def popup_done(self):
        """Popup interaction is done

        The popup_done event is sent out when a popup is dismissed by the
        compositor. The client should destroy the :class:`XdgPopup` object at
        this point.
        """
        self._post_event(1)

    @XdgPopup.event(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def repositioned(self, token):
        """Signal the completion of a repositioned request

        The repositioned event is sent as part of a popup configuration
        sequence, together with :func:`XdgPopup.configure()` and lastly
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` to notify the
        completion of a reposition request.

        The repositioned event is to notify about the completion of a
        :func:`XdgPopup.reposition()` request. The token argument is the token
        passed in the :func:`XdgPopup.reposition()` request.

        Immediately after this event is emitted, :func:`XdgPopup.configure()`
        and :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` will be sent with
        the updated size and position, as well as a new configure serial.

        The client should optionally update the content of the popup, but must
        acknowledge the new popup configuration for the new position to take
        effect. See :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        :param token:
            reposition request token
        :type token:
            `ArgumentType.Uint`
        """
        self._post_event(2, token)


class XdgPopupGlobal(Global):
    interface = XdgPopup


XdgPopup._gen_c()
XdgPopup.proxy_class = XdgPopupProxy
XdgPopup.resource_class = XdgPopupResource
XdgPopup.global_class = XdgPopupGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from dataclasses import dataclass
from typing import Dict, Iterable, List, Optional, Tuple

from .argument import Argument
from .description import Description
from .element import Element
from .printer import Printer


@dataclass(frozen=True)  # type: ignore[misc]
class Method(Element, abc.ABC):
    """Scanner for methods

    Corresponds to event and requests defined on an interface
    """

    name: str
    since: Optional[str]
    description: Optional[Description]
    arg: List[Argument]

    def imports(
        self, interface: str, module_imports: Dict[str, str]
    ) -> List[Tuple[str, str]]:
        """Get the imports required for each of the interfaces

        :param interface:
            The name of the interface that the method is a part of.
        :param module_imports:
            A mapping from the name of an interface in the associated
            module that the interface comes from.
        :return:
            A list of 2-tuples, each specifying the path to an imported
            module and the imported class.
        """
        current_protocol = module_imports[interface]

        imports = []
        for arg in self.arg:
            if arg.interface is None:
                continue

            if arg.interface == interface:
                continue

            import_class = arg.interface_class
            import_protocol = module_imports[arg.interface]

            if current_protocol == import_protocol:
                import_path = ".{}".format(arg.interface)
            else:
                import_path = "..{}".format(import_protocol)

            imports.append((import_path, import_class))

        return imports

    @property
    @abc.abstractmethod
    def method_type(self) -> str:
        pass

    @property
    @abc.abstractmethod
    def method_args(self) -> Iterable[str]:
        pass

    @abc.abstractmethod
    def output_doc_params(self, printer: Printer) -> None:
        pass

    @abc.abstractmethod
    def output_body(self, printer: Printer, opcode: int) -> None:
        pass

    def output(
        self,
        printer: Printer,
        opcode: int,
        in_class: str,
        module_imports: Dict[str, str],
    ) -> None:
        """Generate the output for the given method to the printer"""
        if len(self.arg) > 0:
            printer(f"@{in_class}.{self.method_type}(")
            with printer.indented():
                for arg in self.arg:
                    arg.output(printer)
                if self.since:
                    printer(f"version={self.since},")
            printer(")")
        else:
            if self.since:
                printer(f"@{in_class}.{self.method_type}(version={self.since})")
            else:
                printer(f"@{in_class}.{self.method_type}()")

        # Generate the definition of the method and args
        args = ", ".join(["self"] + list(self.method_args))
        printer("def {}({}):".format(self.name, args))

        with printer.indented():
            # Write the documentation
            self.output_doc(printer)
            # Write out the body of the method
            self.output_body(printer, opcode)

    def output_doc(self, printer: Printer) -> None:
        """Output the documentation for the interface"""
        if self.description:
            self.description.output(printer)
        else:
            printer('"""' + self.name)
        # Parameter and returns documentation
        if self.arg:
            printer()
            self.output_doc_params(printer)
        printer('"""')



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_surface import WlSurface


class WlKeyboard(Interface):
    """Keyboard input device

    The :class:`WlKeyboard` interface represents one or more keyboards
    associated with a seat.
    """

    name = "wl_keyboard"
    version = 7

    class keymap_format(enum.IntEnum):
        no_keymap = 0
        xkb_v1 = 1

    class key_state(enum.IntEnum):
        released = 0
        pressed = 1


class WlKeyboardProxy(Proxy):
    interface = WlKeyboard

    @WlKeyboard.request(version=3)
    def release(self):
        """Release the keyboard object
        """
        self._marshal(0)
        self._destroy()


class WlKeyboardResource(Resource):
    interface = WlKeyboard

    @WlKeyboard.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
    )
    def keymap(self, format, fd, size):
        """Keyboard mapping

        This event provides a file descriptor to the client which can be
        memory-mapped to provide a keyboard mapping description.

        From version 7 onwards, the fd must be mapped with MAP_PRIVATE by the
        recipient, as MAP_SHARED may fail.

        :param format:
            keymap format
        :type format:
            `ArgumentType.Uint`
        :param fd:
            keymap file descriptor
        :type fd:
            `ArgumentType.FileDescriptor`
        :param size:
            keymap size, in bytes
        :type size:
            `ArgumentType.Uint`
        """
        self._post_event(0, format, fd, size)

    @WlKeyboard.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Array),
    )
    def enter(self, serial, surface, keys):
        """Enter event

        Notification that this seat's keyboard focus is on a certain surface.

        The compositor must send the :func:`WlKeyboard.modifiers()` event after
        this event.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface gaining keyboard focus
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param keys:
            the currently pressed keys
        :type keys:
            `ArgumentType.Array`
        """
        self._post_event(1, serial, surface, keys)

    @WlKeyboard.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def leave(self, serial, surface):
        """Leave event

        Notification that this seat's keyboard focus is no longer on a certain
        surface.

        The leave notification is sent before the enter notification for the
        new focus.

        After this event client must assume that all keys, including modifiers,
        are lifted and also it must stop key repeating if there's some going
        on.

        :param serial:
            serial number of the leave event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface that lost keyboard focus
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(2, serial, surface)

    @WlKeyboard.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def key(self, serial, time, key, state):
        """Key event

        A key was pressed or released. The time argument is a timestamp with
        millisecond granularity, with an undefined base.

        The key is a platform-specific key code that can be interpreted by
        feeding it to the keyboard mapping (see the keymap event).

        If this event produces a change in modifiers, then the resulting
        :func:`WlKeyboard.modifiers()` event must be sent after this event.

        :param serial:
            serial number of the key event
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param key:
            key that produced the event
        :type key:
            `ArgumentType.Uint`
        :param state:
            physical state of the key
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(3, serial, time, key, state)

    @WlKeyboard.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def modifiers(self, serial, mods_depressed, mods_latched, mods_locked, group):
        """Modifier and group state

        Notifies clients that the modifier and/or group state has changed, and
        it should update its local state.

        :param serial:
            serial number of the modifiers event
        :type serial:
            `ArgumentType.Uint`
        :param mods_depressed:
            depressed modifiers
        :type mods_depressed:
            `ArgumentType.Uint`
        :param mods_latched:
            latched modifiers
        :type mods_latched:
            `ArgumentType.Uint`
        :param mods_locked:
            locked modifiers
        :type mods_locked:
            `ArgumentType.Uint`
        :param group:
            keyboard layout
        :type group:
            `ArgumentType.Uint`
        """
        self._post_event(4, serial, mods_depressed, mods_latched, mods_locked, group)

    @WlKeyboard.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        version=4,
    )
    def repeat_info(self, rate, delay):
        """Repeat rate and delay

        Informs the client about the keyboard's repeat rate and delay.

        This event is sent as soon as the :class:`WlKeyboard` object has been
        created, and is guaranteed to be received by the client before any key
        press event.

        Negative values for either rate or delay are illegal. A rate of zero
        will disable any repeating (regardless of the value of delay).

        This event can be sent later on as well with a new value if necessary,
        so clients should continue listening for the event past the creation of
        :class:`WlKeyboard`.

        :param rate:
            the rate of repeating keys in characters per second
        :type rate:
            `ArgumentType.Int`
        :param delay:
            delay in milliseconds since key down until repeating starts
        :type delay:
            `ArgumentType.Int`
        """
        self._post_event(5, rate, delay)


class WlKeyboardGlobal(Global):
    interface = WlKeyboard


WlKeyboard._gen_c()
WlKeyboard.proxy_class = WlKeyboardProxy
WlKeyboard.resource_class = WlKeyboardResource
WlKeyboard.global_class = WlKeyboardGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cffi import FFI


###############################################################################
# wayland-version.h
###############################################################################
CDEF = """
#define WAYLAND_VERSION_MAJOR ...
#define WAYLAND_VERSION_MINOR ...
#define WAYLAND_VERSION_MICRO ...
 """

###############################################################################
# wayland-util.h
###############################################################################
# wl_fixed_t handling
CDEF += """
typedef int32_t wl_fixed_t;
static inline double wl_fixed_to_double(wl_fixed_t f);
static inline wl_fixed_t wl_fixed_from_double(double d);
static inline wl_fixed_t wl_fixed_from_int(int i);
"""

# Event/request dispatching structs
CDEF += """
struct wl_message {
    const char *name;
    const char *signature;
    const struct wl_interface **types;
};
struct wl_interface {
    const char *name;
    int version;
    int method_count;
    const struct wl_message *methods;
    int event_count;
    const struct wl_message *events;
};
union wl_argument {
    int32_t i; /**< signed integer */
    uint32_t u; /**< unsigned integer */
    wl_fixed_t f; /**< fixed point */
    const char *s; /**< string */
    struct wl_object *o; /**< object */
    uint32_t n; /**< new_id */
    struct wl_array *a; /**< array */
    int32_t h; /**< file descriptor */
};
"""

# wl_array methods
CDEF += """
struct wl_array {
    size_t size;
    size_t alloc;
    void *data;
};
"""

# wl_list methods
CDEF += """
struct wl_list {
    struct wl_list *prev;
    struct wl_list *next;
};

void wl_list_remove(struct wl_list *elm);
int wl_list_empty(const struct wl_list *list);
"""

# Dispatcher callback
CDEF += """
typedef int (*wl_dispatcher_func_t)(const void *, void *, uint32_t,
                                    const struct wl_message *,
                                    union wl_argument *);
"""

###############################################################################
# wayland-client.h
###############################################################################
# wl_eventqueue methods
CDEF += """
void wl_event_queue_destroy(struct wl_event_queue *queue);
"""

# wl_proxy methods
CDEF += """
void wl_proxy_marshal_array(struct wl_proxy *p, uint32_t opcode,
                            union wl_argument *args);
struct wl_proxy *wl_proxy_create(struct wl_proxy *factory,
                                 const struct wl_interface *interface);
struct wl_proxy *
wl_proxy_marshal_array_constructor(struct wl_proxy *proxy,
                                   uint32_t opcode, union wl_argument *args,
                                   const struct wl_interface *interface);

void wl_proxy_destroy(struct wl_proxy *proxy);
int wl_proxy_add_dispatcher(struct wl_proxy *proxy,
                            wl_dispatcher_func_t dispatcher_func,
                            const void * dispatcher_data, void *data);
void wl_proxy_set_user_data(struct wl_proxy *proxy, void *user_data);
void *wl_proxy_get_user_data(struct wl_proxy *proxy);
"""

# wl_display methods
CDEF += """
struct wl_display* wl_display_connect(const char *name);
struct wl_display* wl_display_connect_to_fd(int fd);
void wl_display_disconnect(struct wl_display *display);
int wl_display_get_fd(struct wl_display *display);
int wl_display_dispatch(struct wl_display *display);
int wl_display_dispatch_pending(struct wl_display *display);
int wl_display_dispatch_queue(struct wl_display *display,
                              struct wl_event_queue *queue);
int wl_display_dispatch_queue_pending(struct wl_display *display,
                                      struct wl_event_queue *queue);
int wl_display_roundtrip(struct wl_display *display);
int wl_display_roundtrip_queue(struct wl_display *display,
                               struct wl_event_queue *queue);
int wl_display_get_error(struct wl_display *display);
int wl_display_read_events(struct wl_display *display);
int wl_display_prepare_read(struct wl_display *display);
int wl_display_prepare_read_queue(struct wl_display *display,
                                  struct wl_event_queue *queue);
int wl_display_flush(struct wl_display *display);
struct wl_event_queue *wl_display_create_queue(struct wl_display *display);
"""

###############################################################################
# wayland-server.h
###############################################################################
# wl_eventloop enum
CDEF += """
enum {
    WL_EVENT_READABLE = ...,
    WL_EVENT_WRITABLE = ...,
    WL_EVENT_HANGUP = ...,
    WL_EVENT_ERROR = ...
};
"""

# wl_listener struct
CDEF += """
typedef void (*wl_notify_func_t)(struct wl_listener *listener, void *data);
struct wl_listener {
    struct wl_list link;
    wl_notify_func_t notify;
};
"""

CDEF += """
struct wl_signal {
    struct wl_list listener_list;
};

void wl_signal_add(struct wl_signal *signal, struct wl_listener *listener);
void wl_signal_emit(struct wl_signal *signal, void *data);
"""

# wl_eventloop callbacks and methods
CDEF += """
typedef int (*wl_event_loop_fd_func_t)(int fd, uint32_t mask, void *data);
typedef int (*wl_event_loop_timer_func_t)(void *data);
typedef int (*wl_event_loop_signal_func_t)(int signal_number, void *data);
typedef void (*wl_event_loop_idle_func_t)(void *data);

void wl_event_loop_add_destroy_listener(struct wl_event_loop *loop,
                                        struct wl_listener * listener);

struct wl_event_loop *wl_event_loop_create(void);
void wl_event_loop_destroy(struct wl_event_loop *loop);
struct wl_event_source *wl_event_loop_add_fd(struct wl_event_loop *loop,
                                             int fd, uint32_t mask,
                                             wl_event_loop_fd_func_t func,
                                             void *data);
int wl_event_source_fd_update(struct wl_event_source *source, uint32_t mask);
struct wl_event_source *wl_event_loop_add_timer(struct wl_event_loop *loop,
                                                wl_event_loop_timer_func_t func,
                                                void *data);
struct wl_event_source *
wl_event_loop_add_signal(struct wl_event_loop *loop,
                         int signal_number,
                         wl_event_loop_signal_func_t func,
                         void *data);

int wl_event_loop_dispatch(struct wl_event_loop *loop, int timeout);
void wl_event_loop_dispatch_idle(struct wl_event_loop *loop);
struct wl_event_source *wl_event_loop_add_idle(struct wl_event_loop *loop,
                                               wl_event_loop_idle_func_t func,
                                               void *data);
int wl_event_loop_get_fd(struct wl_event_loop *loop);
"""

# wl_event_source methods
CDEF += """
int wl_event_source_timer_update(struct wl_event_source *source,
                                 int ms_delay);
int wl_event_source_remove(struct wl_event_source *source);
void wl_event_source_check(struct wl_event_source *source);
"""

# wl_display methods
CDEF += """
struct wl_display * wl_display_create(void);
void wl_display_destroy(struct wl_display *display);
struct wl_event_loop *wl_display_get_event_loop(struct wl_display *display);
int wl_display_add_socket(struct wl_display *display, const char *name);
const char *wl_display_add_socket_auto(struct wl_display *display);
void wl_display_terminate(struct wl_display *display);
void wl_display_run(struct wl_display *display);

uint32_t wl_display_get_serial(struct wl_display *display);
uint32_t wl_display_next_serial(struct wl_display *display);
void wl_display_destroy_clients(struct wl_display *display);
void wl_display_flush_clients(struct wl_display *display);

int wl_display_init_shm(struct wl_display *display);
uint32_t *wl_display_add_shm_format(struct wl_display *display, uint32_t format);
"""

# wl_global methods
CDEF += """
typedef void (*wl_global_bind_func_t)(struct wl_client *client, void *data,
                                      uint32_t version, uint32_t id);
struct wl_global *wl_global_create(struct wl_display *display,
                                   const struct wl_interface *interface,
                                   int version,
                                   void *data, wl_global_bind_func_t bind);
void wl_global_destroy(struct wl_global *global);
"""

# wl_client methods
CDEF += """
struct wl_client;
struct wl_client *wl_client_create(struct wl_display *display, int fd);
void wl_client_destroy(struct wl_client *client);
void wl_client_flush(struct wl_client *client);

typedef int pid_t;
typedef unsigned int uid_t;
typedef unsigned int gid_t;
void wl_client_get_credentials(struct wl_client *client,
    pid_t *pid, uid_t *uid, gid_t *gid);

void wl_client_add_destroy_listener(struct wl_client *client,
                                    struct wl_listener *listener);

struct wl_resource *
wl_client_get_object(struct wl_client *client, uint32_t id);
"""

# wl_resource methods
CDEF += """
typedef void (*wl_resource_destroy_func_t)(struct wl_resource *resource);

void wl_resource_post_event_array(struct wl_resource *resource,
                                  uint32_t opcode, union wl_argument *args);

void wl_resource_post_error(struct wl_resource *resource,
                            uint32_t code, const char *msg, ...);

struct wl_resource *
wl_resource_create(struct wl_client *client,
                   const struct wl_interface *interface,
                   int version, uint32_t id);
void
wl_resource_set_dispatcher(struct wl_resource *resource,
                           wl_dispatcher_func_t dispatcher,
                           const void *implementation,
                           void *data,
                           wl_resource_destroy_func_t destroy);

void
wl_resource_destroy(struct wl_resource *resource);
uint32_t
wl_resource_get_id(struct wl_resource *resource);
void *
wl_resource_get_user_data(struct wl_resource *resource);
int
wl_resource_get_version(struct wl_resource *resource);

void
wl_resource_add_destroy_listener(struct wl_resource *resource,
                                 struct wl_listener * listener);
"""

# anonymous file methods (from Weston)
CDEF += """
int
os_create_anonymous_file(int size);
"""

# cffi callback functions
CDEF += """
extern "Python" int dispatcher_func(const void *, void *, uint32_t, const struct wl_message *, union wl_argument *);
extern "Python" void resource_destroy_func(struct wl_resource *);
extern "Python" int event_loop_fd_func(int, uint32_t, void *);
extern "Python" int event_loop_signal_func(int, void *);
extern "Python" int event_loop_timer_func(void *);
extern "Python" void event_loop_idle_func(void *);
extern "Python" void global_bind_func(struct wl_client *, void *, uint32_t, uint32_t);
extern "Python" void notify_func(struct wl_listener *, void *);

struct wl_listener_container {
    void *handle;
    struct wl_listener destroy_listener;
};
"""

SOURCE = """
#include <wayland-client-core.h>
#include <wayland-server-core.h>
#include <wayland-version.h>

#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
"""

SOURCE += """
struct wl_listener_container {
    void *handle;
    struct wl_listener destroy_listener;
};
"""

SOURCE += """
/* This code is taken from Weston (MIT licensed) to provide access to anonymous
 * files with CLOEXEC set
 * Copyright (c) 2012 Collabora, Ltd.
 * http://cgit.freedesktop.org/wayland/weston/tree/shared/os-compatibility.c?id=1.8.0
 */

static int
set_cloexec_or_close(int fd)
{
    long flags;

    if (fd == -1)
        return -1;

    flags = fcntl(fd, F_GETFD);
    if (flags == -1)
        goto err;

    if (fcntl(fd, F_SETFD, flags | FD_CLOEXEC) == -1)
        goto err;

    return fd;

err:
    close(fd);
    return -1;
}

static int
create_tmpfile_cloexec(char *tmpname)
{
    int fd;

#ifdef HAVE_MKOSTEMP
    fd = mkostemp(tmpname, O_CLOEXEC);
    if (fd >= 0)
        unlink(tmpname);
#else
    fd = mkstemp(tmpname);
    if (fd >= 0) {
        fd = set_cloexec_or_close(fd);
        unlink(tmpname);
    }
#endif

    return fd;
}

int
os_create_anonymous_file(off_t size)
{
    static const char template[] = "/weston-shared-XXXXXX";
    const char *path;
    char *name;
    int fd;
    int ret;

    path = getenv("XDG_RUNTIME_DIR");
    if (!path) {
        errno = ENOENT;
        return -1;
    }

    name = malloc(strlen(path) + sizeof(template));
    if (!name)
        return -1;

    strcpy(name, path);
    strcat(name, template);

    fd = create_tmpfile_cloexec(name);

    free(name);

    if (fd < 0)
        return -1;

#ifdef HAVE_POSIX_FALLOCATE
    ret = posix_fallocate(fd, 0, size);
    if (ret != 0) {
        close(fd);
        errno = ret;
        return -1;
    }
#else
    ret = ftruncate(fd, size);
    if (ret < 0) {
        close(fd);
        return -1;
    }
#endif

    return fd;
}
"""

ffi_builder = FFI()
ffi_builder.set_source(
    "pywayland._ffi", SOURCE, libraries=["wayland-client", "wayland-server"]
)
ffi_builder.cdef(CDEF)


if __name__ == "__main__":
    ffi_builder.compile()



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZxdgToplevelDecorationV1(Interface):
    """Decoration object for a toplevel surface

    The decoration object allows the compositor to toggle server-side window
    decorations for a toplevel surface. The client can request to switch to
    another mode.

    The xdg_toplevel_decoration object must be destroyed before its
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.
    """

    name = "zxdg_toplevel_decoration_v1"
    version = 1

    class error(enum.IntEnum):
        unconfigured_buffer = 0
        already_constructed = 1
        orphaned = 2

    class mode(enum.IntEnum):
        client_side = 1
        server_side = 2


class ZxdgToplevelDecorationV1Proxy(Proxy):
    interface = ZxdgToplevelDecorationV1

    @ZxdgToplevelDecorationV1.request()
    def destroy(self):
        """Destroy the decoration object

        Switch back to a mode without any server-side decorations at the next
        commit.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgToplevelDecorationV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_mode(self, mode):
        """Set the decoration mode

        Set the toplevel surface decoration mode. This informs the compositor
        that the client prefers the provided decoration mode.

        After requesting a decoration mode, the compositor will respond by
        emitting an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event. The client
        should then update its content, drawing it without decorations if the
        received mode is server-side decorations. The client must also
        acknowledge the configure when committing the new content (see
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>`).

        The compositor can decide not to use the client's mode and enforce a
        different mode instead.

        Clients whose decoration mode depend on the
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` state may send a
        set_mode request in response to an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event and wait for
        the next :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event to prevent
        unwanted state. Such clients are responsible for preventing configure
        loops and must make sure not to send multiple successive set_mode
        requests with the same decoration mode.

        :param mode:
            the decoration mode
        :type mode:
            `ArgumentType.Uint`
        """
        self._marshal(1, mode)

    @ZxdgToplevelDecorationV1.request()
    def unset_mode(self):
        """Unset the decoration mode

        Unset the toplevel surface decoration mode. This informs the compositor
        that the client doesn't prefer a particular decoration mode.

        This request has the same semantics as set_mode.
        """
        self._marshal(2)


class ZxdgToplevelDecorationV1Resource(Resource):
    interface = ZxdgToplevelDecorationV1

    @ZxdgToplevelDecorationV1.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, mode):
        """Suggest a surface change

        The configure event asks the client to change its decoration mode. The
        configured state should not be applied immediately. Clients must send
        an ack_configure in response to this event. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` and
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        A configure event can be sent at any time. The specified mode must be
        obeyed by the client.

        :param mode:
            the decoration mode
        :type mode:
            `ArgumentType.Uint`
        """
        self._post_event(0, mode)


class ZxdgToplevelDecorationV1Global(Global):
    interface = ZxdgToplevelDecorationV1


ZxdgToplevelDecorationV1._gen_c()
ZxdgToplevelDecorationV1.proxy_class = ZxdgToplevelDecorationV1Proxy
ZxdgToplevelDecorationV1.resource_class = ZxdgToplevelDecorationV1Resource
ZxdgToplevelDecorationV1.global_class = ZxdgToplevelDecorationV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from ..wayland import WlSurface
from ..xdg_shell import XdgPopup
from ..xdg_shell import XdgSurface


class XdgShell(Interface):
    """Create desktop-style surfaces

    :class:`XdgShell` allows clients to turn a
    :class:`~pywayland.protocol.wayland.WlSurface` into a "real window" which
    can be dragged, resized, stacked, and moved around by the user. Everything
    about this interface is suited towards traditional desktop environments.
    """

    name = "xdg_shell"
    version = 1

    class version_(enum.IntEnum):
        current = 5

    class error(enum.IntEnum):
        role = 0
        defunct_surfaces = 1
        not_the_topmost_popup = 2
        invalid_popup_parent = 3


class XdgShellProxy(Proxy):
    interface = XdgShell

    @XdgShell.request()
    def destroy(self):
        """Destroy :class:`XdgShell`

        Destroy this :class:`XdgShell` object.

        Destroying a bound :class:`XdgShell` object while there are surfaces
        still alive created by this :class:`XdgShell` object instance is
        illegal and will result in a protocol error.
        """
        self._marshal(0)
        self._destroy()

    @XdgShell.request(
        Argument(ArgumentType.Int),
    )
    def use_unstable_version(self, version):
        """Enable use of this unstable version

        Negotiate the unstable version of the interface.  This mechanism is in
        place to ensure client and server agree on the unstable versions of the
        protocol that they speak or exit cleanly if they don't agree.  This
        request will go away once the xdg-shell protocol is stable.

        :param version:
        :type version:
            `ArgumentType.Int`
        """
        self._marshal(1, version)

    @XdgShell.request(
        Argument(ArgumentType.NewId, interface=XdgSurface),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_xdg_surface(self, surface):
        """Create a shell surface from a surface

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for
        the given surface and gives it the
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` role. A
        :class:`~pywayland.protocol.wayland.WlSurface` can only be given an
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` role once. If
        get_xdg_surface is called with a
        :class:`~pywayland.protocol.wayland.WlSurface` that already has an
        active :class:`~pywayland.protocol.xdg_shell.XdgSurface` associated
        with it, or if it had any other role, an error is raised.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgSurface` is and
        how it is used.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgSurface`
        """
        id = self._marshal_constructor(2, XdgSurface, surface)
        return id

    @XdgShell.request(
        Argument(ArgumentType.NewId, interface=XdgPopup),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def get_xdg_popup(self, surface, parent, seat, serial, x, y):
        """Create a popup for a surface

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgPopup` for the
        given surface and gives it the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` role. A
        :class:`~pywayland.protocol.wayland.WlSurface` can only be given an
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` role once. If
        get_xdg_popup is called with a
        :class:`~pywayland.protocol.wayland.WlSurface` that already has an
        active :class:`~pywayland.protocol.xdg_shell.XdgPopup` associated with
        it, or if it had any other role, an error is raised.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` for more details about
        what an :class:`~pywayland.protocol.xdg_shell.XdgPopup` is and how it
        is used.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param parent:
        :type parent:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgPopup`
        """
        id = self._marshal_constructor(3, XdgPopup, surface, parent, seat, serial, x, y)
        return id

    @XdgShell.request(
        Argument(ArgumentType.Uint),
    )
    def pong(self, serial):
        """Respond to a ping event

        A client must respond to a ping event with a pong request or the client
        may be deemed unresponsive.

        :param serial:
            serial of the ping event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(4, serial)


class XdgShellResource(Resource):
    interface = XdgShell

    @XdgShell.event(
        Argument(ArgumentType.Uint),
    )
    def ping(self, serial):
        """Check if the client is alive

        The ping event asks the client if it's still alive. Pass the serial
        specified in the event back to the compositor by sending a "pong"
        request back with the specified serial.

        Compositors can use this to determine if the client is still alive.
        It's unspecified what will happen if the client doesn't respond to the
        ping request, or in what timeframe. Clients should try to respond in a
        reasonable amount of time.

        A compositor is free to ping in any way it wants, but a client must
        always respond to any :class:`XdgShell` object it created.

        :param serial:
            pass this to the pong request
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class XdgShellGlobal(Global):
    interface = XdgShell


XdgShell._gen_c()
XdgShell.proxy_class = XdgShellProxy
XdgShell.resource_class = XdgShellResource
XdgShell.global_class = XdgShellGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .xdg_popup import XdgPopup
from .xdg_positioner import XdgPositioner
from .xdg_toplevel import XdgToplevel


class XdgSurface(Interface):
    """Desktop user interface surface base interface

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides a base set of functionality required to construct user
    interface elements requiring management by the compositor, such as toplevel
    windows, menus, etc. The types of functionality are split into
    :class:`XdgSurface` roles.

    Creating an :class:`XdgSurface` does not set the role for a
    :class:`~pywayland.protocol.wayland.WlSurface`. In order to map an
    :class:`XdgSurface`, the client must create a role-specific object using,
    e.g., get_toplevel, get_popup. The
    :class:`~pywayland.protocol.wayland.WlSurface` for any given
    :class:`XdgSurface` can have at most one role, and may not be assigned any
    role not based on :class:`XdgSurface`.

    A role must be assigned before any other requests are made to the
    :class:`XdgSurface` object.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the :class:`XdgSurface`
    state to take effect.

    Creating an :class:`XdgSurface` from a
    :class:`~pywayland.protocol.wayland.WlSurface` which has a buffer attached
    or committed is a client error, and any attempts by a client to attach or
    manipulate a buffer prior to the first :func:`XdgSurface.configure()` call
    must also be treated as errors.

    After creating a role-specific object and setting it up, the client must
    perform an initial commit without any buffer attached. The compositor will
    reply with an :func:`XdgSurface.configure()` event. The client must
    acknowledge it and is then allowed to attach a buffer to map the surface.

    Mapping an :class:`XdgSurface`-based role surface is defined as making it
    possible for the surface to be shown by the compositor. Note that a mapped
    surface is not guaranteed to be visible once it is mapped.

    For an :class:`XdgSurface` to be mapped by the compositor, the following
    conditions must be met: (1) the client has assigned an
    :class:`XdgSurface`-based role to the surface (2) the client has set and
    committed the :class:`XdgSurface` state and the     role-dependent state to
    the surface (3) the client has committed a buffer to the surface

    A newly-unmapped surface is considered to have met condition (1) out of the
    3 required conditions for mapping a surface if its role surface has not
    been destroyed.
    """

    name = "xdg_surface"
    version = 3

    class error(enum.IntEnum):
        not_constructed = 1
        already_constructed = 2
        unconfigured_buffer = 3


class XdgSurfaceProxy(Proxy):
    interface = XdgSurface

    @XdgSurface.request()
    def destroy(self):
        """Destroy the :class:`XdgSurface`

        Destroy the :class:`XdgSurface` object. An :class:`XdgSurface` must
        only be destroyed after its role object has been destroyed.
        """
        self._marshal(0)
        self._destroy()

    @XdgSurface.request(
        Argument(ArgumentType.NewId, interface=XdgToplevel),
    )
    def get_toplevel(self):
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        object for the given :class:`XdgSurface` and gives the associated
        :class:`~pywayland.protocol.wayland.WlSurface` the
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` role.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` for more details
        about what an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` is and
        how it is used.

        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        """
        id = self._marshal_constructor(1, XdgToplevel)
        return id

    @XdgSurface.request(
        Argument(ArgumentType.NewId, interface=XdgPopup),
        Argument(ArgumentType.Object, interface=XdgSurface, nullable=True),
        Argument(ArgumentType.Object, interface=XdgPositioner),
    )
    def get_popup(self, parent, positioner):
        """Assign the :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role

        This creates an :class:`~pywayland.protocol.xdg_shell.XdgPopup` object
        for the given :class:`XdgSurface` and gives the associated
        :class:`~pywayland.protocol.wayland.WlSurface` the
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` role.

        If null is passed as a parent, a parent surface must be specified using
        some other protocol, before committing the initial state.

        See the documentation of
        :class:`~pywayland.protocol.xdg_shell.XdgPopup` for more details about
        what an :class:`~pywayland.protocol.xdg_shell.XdgPopup` is and how it
        is used.

        :param parent:
        :type parent:
            :class:`XdgSurface` or `None`
        :param positioner:
        :type positioner:
            :class:`~pywayland.protocol.xdg_shell.XdgPositioner`
        :returns:
            :class:`~pywayland.protocol.xdg_shell.XdgPopup`
        """
        id = self._marshal_constructor(2, XdgPopup, parent, positioner)
        return id

    @XdgSurface.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_window_geometry(self, x, y, width, height):
        """Set the new window geometry

        The window geometry of a surface is its "visible bounds" from the
        user's perspective. Client-side decorations often have invisible
        portions like drop-shadows which should be ignored for the purposes of
        aligning, placing and constraining windows.

        The window geometry is double buffered, and will be applied at the time
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` of the corresponding
        :class:`~pywayland.protocol.wayland.WlSurface` is called.

        When maintaining a position, the compositor should treat the (x, y)
        coordinate of the window geometry as the top left corner of the window.
        A client changing the (x, y) window geometry coordinate should in
        general not alter the position of the window.

        Once the window geometry of the surface is set, it is not possible to
        unset it, and it will remain the same until set_window_geometry is
        called again, even if a new subsurface or buffer is attached.

        If never set, the value is the full bounds of the surface, including
        any subsurfaces. This updates dynamically on every commit. This unset
        is meant for extremely simple clients.

        The arguments are given in the surface-local coordinate space of the
        :class:`~pywayland.protocol.wayland.WlSurface` associated with this
        :class:`XdgSurface`.

        The width and height must be greater than zero. Setting an invalid size
        will raise an error. When applied, the effective window geometry will
        be the set window geometry clamped to the bounding rectangle of the
        combined geometry of the surface of the :class:`XdgSurface` and the
        associated subsurfaces.

        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(3, x, y, width, height)

    @XdgSurface.request(
        Argument(ArgumentType.Uint),
    )
    def ack_configure(self, serial):
        """Ack a configure event

        When a configure event is received, if a client commits the surface in
        response to the configure event, then the client must make an
        ack_configure request sometime before the commit request, passing along
        the serial of the configure event.

        For instance, for toplevel surfaces the compositor might use this
        information to move a surface to the top left only when the client has
        drawn itself for the maximized or fullscreen state.

        If the client receives multiple configure events before it can respond
        to one, it only has to ack the last configure event.

        A client is not required to commit immediately after sending an
        ack_configure request - it may even ack_configure several times before
        its next surface commit.

        A client may send multiple ack_configure requests before committing,
        but only the last request sent before a commit indicates which
        configure event the client really is responding to.

        :param serial:
            the serial from the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(4, serial)


class XdgSurfaceResource(Resource):
    interface = XdgSurface

    @XdgSurface.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, serial):
        """Suggest a surface change

        The configure event marks the end of a configure sequence. A configure
        sequence is a set of one or more events configuring the state of the
        :class:`XdgSurface`, including the final :func:`XdgSurface.configure()`
        event.

        Where applicable, :class:`XdgSurface` surface roles will during a
        configure sequence extend this event as a latched state sent as events
        before the :func:`XdgSurface.configure()` event. Such events should be
        considered to make up a set of atomically applied configuration states,
        where the :func:`XdgSurface.configure()` commits the accumulated state.

        Clients should arrange their surface for the new states, and then send
        an ack_configure request with the serial sent in this configure event
        at some point before committing the new surface.

        If the client receives multiple configure events before it can respond
        to one, it is free to discard all but the last event it received.

        :param serial:
            serial of the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)


class XdgSurfaceGlobal(Global):
    interface = XdgSurface


XdgSurface._gen_c()
XdgSurface.proxy_class = XdgSurfaceProxy
XdgSurface.resource_class = XdgSurfaceResource
XdgSurface.global_class = XdgSurfaceGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, TYPE_CHECKING

from pywayland import ffi, lib
from pywayland.utils import ensure_valid
from .eventloop import EventLoop

if TYPE_CHECKING:
    from typing_extensions import Literal


def _full_display_gc(ptr: "ffi.DisplayCData") -> None:
    """Destroy the Display cdata pointer, but only after destroying the clients"""
    lib.wl_display_destroy_clients(ptr)
    lib.wl_display_destroy(ptr)


class Display:
    """Create a Wayland Display object"""

    def __init__(self, ptr=None) -> None:
        if ptr is None:
            ptr = lib.wl_display_create()
            if ptr == ffi.NULL:
                raise MemoryError("Unable to create wl_display object")

        self._ptr = ffi.gc(ptr, _full_display_gc)

    def __enter__(self) -> "Display":
        """Use the Display in a context manager, which automatically destroys the Display"""
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> "Literal[False]":
        """Destroy the used display"""
        self.destroy()
        return False

    @property
    def destroyed(self) -> bool:
        """Returns if the display has been destroyed"""
        return self._ptr is None

    def destroy(self) -> None:
        """Destroy Wayland display object.

        This function emits the :class:`Display` destroy signal, releases all
        the sockets added to this display, free's all the globals associated
        with this display, free's memory of additional shared memory formats
        and destroy the display object.

        .. seealso::

            :meth:`Display.add_destroy_listener()`
        """
        if self._ptr is not None:
            ffi.release(self._ptr)
            self._ptr = None

    @ensure_valid
    def add_socket(self, name: Optional[str] = None) -> str:
        """Add a socket to Wayland display for the clients to connect.

        This adds a Unix socket to Wayland display which can be used by clients
        to connect to Wayland display.

        If `None` is passed as name, then it would look for `WAYLAND_DISPLAY`
        environment variable for the socket name. If `WAYLAND_DISPLAY` is not
        set, then default `wayland-0` is used.

        The Unix socket will be created in the directory pointed to by
        environment variable `XDG_RUNTIME_DIR`. If `XDG_RUNTIME_DIR` is not
        set, then this function throws an exception.

        The length of socket path, i.e., the path set in `XDG_RUNTIME_DIR` and
        the socket name, must not exceed the maxium length of a Unix socket
        path.  The function also fails if the user do not have write permission
        in the `XDG_RUNTIME_DIR` path or if the socket name is already in use.

        :param name: Name of the Unix socket.
        :type name: string or None
        """
        if name is None:
            name_ptr = lib.wl_display_add_socket_auto(self._ptr)
            if name_ptr == ffi.NULL:
                raise RuntimeError("Unable to create socket")
            name = ffi.string(name_ptr)
        else:
            ret = lib.wl_display_add_socket(self._ptr, name.encode())

            if ret == -1:
                # TODO: raise better
                raise Exception()

        return name

    @ensure_valid
    def get_serial(self) -> int:
        """Get the current serial number

        This function returns the most recent serial number, but does not
        increment it.
        """
        return lib.wl_display_get_serial(self._ptr)

    @ensure_valid
    def next_serial(self) -> int:
        """Get the next serial

        This function increments the display serial number and returns the new
        value.
        """
        return lib.wl_display_next_serial(self._ptr)

    @ensure_valid
    def get_event_loop(self) -> EventLoop:
        """Get the event loop for the display

        :returns: The :class:`~pywayland.server.EventLoop` for the Display
        """
        return EventLoop(self)

    @ensure_valid
    def terminate(self) -> None:
        """Stop the display from running"""
        lib.wl_display_terminate(self._ptr)

    @ensure_valid
    def run(self) -> None:
        """Run the display"""
        lib.wl_display_run(self._ptr)

    @ensure_valid
    def init_shm(self) -> None:
        """Initialize shm for this display"""
        ret = lib.wl_display_init_shm(self._ptr)
        if ret == -1:
            raise MemoryError("Unable to create shm for display")

    @ensure_valid
    def add_shm_format(self, shm_format) -> None:
        """Add support for a Shm pixel format

        Add the specified :class:`~pywayland.protocol.wayland.WlShm.format`
        format to the list of formats the
        :class:`~pywayland.protocol.wayland.WlShm` object advertises when a
        client binds to it.  Adding a format to the list means that clients
        will know that the compositor supports this format and may use it for
        creating :class:`~pywayland.protocol.wayland.WlShm` buffers.  The
        compositor must be able to handle the pixel format when a client
        requests it.

        The compositor by default supports ``WL_SHM_FORMAT_ARGB8888`` and
        ``WL_SHM_FORMAT_XRGB8888``.

        :param shm_format: The shm pixel format to advertise
        :type shm_format: :class:`~pywayland.protocol.wayland.WlShm.format`
        """
        lib.wl_display_add_shm_format(self._ptr, shm_format.value)

    @ensure_valid
    def flush_clients(self) -> None:
        """Flush client connections"""
        lib.wl_display_flush_clients(self._ptr)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .zxdg_exported_v1 import ZxdgExportedV1


class ZxdgExporterV1(Interface):
    """Interface for exporting surfaces

    A global interface used for exporting surfaces that can later be imported
    using xdg_importer.
    """

    name = "zxdg_exporter_v1"
    version = 1


class ZxdgExporterV1Proxy(Proxy):
    interface = ZxdgExporterV1

    @ZxdgExporterV1.request()
    def destroy(self):
        """Destroy the xdg_exporter object

        Notify the compositor that the xdg_exporter object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgExporterV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgExportedV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def export(self, surface):
        """Export a surface

        The export request exports the passed surface so that it can later be
        imported via xdg_importer. When called, a new xdg_exported object will
        be created and xdg_exported.handle will be sent immediately. See the
        corresponding interface and event for details.

        A surface may be exported multiple times, and each exported handle may
        be used to create an xdg_imported multiple times. Only
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` surfaces may be
        exported.

        :param surface:
            the surface to export
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v1.ZxdgExportedV1`
            -- the new xdg_exported object
        """
        id = self._marshal_constructor(1, ZxdgExportedV1, surface)
        return id


class ZxdgExporterV1Resource(Resource):
    interface = ZxdgExporterV1


class ZxdgExporterV1Global(Global):
    interface = ZxdgExporterV1


ZxdgExporterV1._gen_c()
ZxdgExporterV1.proxy_class = ZxdgExporterV1Proxy
ZxdgExporterV1.resource_class = ZxdgExporterV1Resource
ZxdgExporterV1.global_class = ZxdgExporterV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Callable, Iterable, List, Optional, Tuple
from weakref import WeakKeyDictionary

from pywayland import ffi, lib
from .argument import Argument, ArgumentType

weakkeydict: WeakKeyDictionary = WeakKeyDictionary()


class Message:
    """Wrapper class for `wl_message` structs

    Base class that correspond to the methods defined on an interface in the
    wayland.xml protocol, and are generated by the scanner.  Subclasses specify
    the type of method, whether it is a server-side or client-side method.

    :param func:
        The function that is represented by the message
    :type func:
        `function`
    :param signature:
        The signature of the arguments of the message
    :type signature:
        `string`
    :param types:
        List of the types of any objects included in the argument list, None if
        otherwise.
    :type types:
        `list`
    """

    def __init__(
        self, func: Callable, arguments: List[Argument], version: Optional[int]
    ) -> None:
        self.py_func = func

        self.name = func.__name__.strip("_")
        self.arguments = arguments
        self.version = version

    @property
    def _marshaled_arguments(self) -> Iterable[Argument]:
        for arg in self.arguments:
            if arg.interface is None and arg.argument_type == ArgumentType.NewId:
                yield Argument(ArgumentType.String)
                yield Argument(ArgumentType.Uint)
            yield arg

    def build_message_struct(self, wl_message_struct) -> Tuple:
        """Bulid the wl_message struct for this message

        :param wl_message_struct:
            The wl_message cdata struct to use to build the message struct.
        :return:
            A tuple of elements which must be kept alive for the message struct
            to remain valid.
        """
        signature = "".join(argument.signature for argument in self.arguments)
        if self.version is not None:
            signature = f"{self.version}{signature}"

        wl_message_struct.name = name = ffi.new("char[]", self.name.encode())
        wl_message_struct.signature = cdata_signature = ffi.new(
            "char[]", signature.encode()
        )

        wl_message_struct.types = types = ffi.new(
            "struct wl_interface* []", len(list(self._marshaled_arguments))
        )

        for index, argument in enumerate(self._marshaled_arguments):
            if argument.interface is None:
                types[index] = ffi.NULL
            else:
                assert argument.interface._ptr is not None
                types[index] = argument.interface._ptr

        return name, cdata_signature, types

    def c_to_arguments(self, args_ptr):
        """Create a list of arguments

        Generate the arguments of the method from a CFFI cdata array of
        `wl_argument` structs that correspond to the arguments of the method as
        specified by the method signature.

        :param args_ptr: Input arguments
        :type args_ptr: cdata `union wl_argument []`
        :returns: list of args
        """
        args = []
        for i, argument in enumerate(self.arguments):
            arg_ptr = args_ptr[i]

            # Match numbers (int, unsigned, float, file descriptor)
            if argument.argument_type == ArgumentType.Int:
                args.append(arg_ptr.i)
            elif argument.argument_type == ArgumentType.Uint:
                args.append(arg_ptr.u)
            elif argument.argument_type == ArgumentType.Fixed:
                f = lib.wl_fixed_to_double(arg_ptr.f)
                args.append(f)
            elif argument.argument_type == ArgumentType.FileDescriptor:
                args.append(arg_ptr.h)
            elif argument.argument_type == ArgumentType.String:
                if arg_ptr == ffi.NULL:
                    if not argument.nullable:
                        raise Exception
                    args.append(None)
                else:
                    args.append(ffi.string(arg_ptr.s).decode())
            elif argument.argument_type == ArgumentType.Object:
                if arg_ptr.o == ffi.NULL:
                    if not argument.nullable:
                        message = "Got null object parsing arguments for '{}' message, may already be destroyed".format(
                            self.name
                        )
                        raise RuntimeError(message)
                    args.append(None)
                else:
                    iface = argument.interface
                    proxy_ptr = ffi.cast("struct wl_proxy *", arg_ptr.o)
                    obj = iface.registry.get(proxy_ptr)
                    if obj is None:
                        raise RuntimeError(
                            "Unable to get object for {}, was it garbage collected?".format(
                                proxy_ptr
                            )
                        )
                    args.append(obj)
            elif argument.argument_type == ArgumentType.NewId:
                # TODO
                raise NotImplementedError
            elif argument.argument_type == ArgumentType.Array:
                array_ptr = arg_ptr.a
                args.append(ffi.buffer(array_ptr.data, array_ptr.size)[:])
            else:
                raise Exception(f"Bad argument: {argument}")

        return args

    def arguments_to_c(self, *args):
        """Create an array of `wl_argument` C structs

        Generate the CFFI cdata array of `wl_argument` structs that correspond
        to the arguments of the method as specified by the method signature.

        :param args: Input arguments
        :type args: `list`
        :returns: cdata `union wl_argument []` of args
        """
        nargs = len(list(self._marshaled_arguments))
        args_ptr = ffi.new("union wl_argument []", nargs)

        arg_iter = iter(args)
        refs = []
        for i, argument in enumerate(self._marshaled_arguments):
            # New id (set to null for now, will be assigned on marshal)
            # Then, continue so we don't consume an arg
            if argument.argument_type == ArgumentType.NewId:
                args_ptr[i].o = ffi.NULL
                continue

            arg = next(arg_iter)
            # Match numbers (int, unsigned, float, file descriptor)
            if argument.argument_type == ArgumentType.Int:
                args_ptr[i].i = arg
            elif argument.argument_type == ArgumentType.Uint:
                args_ptr[i].u = arg
            elif argument.argument_type == ArgumentType.Fixed:
                if isinstance(arg, int):
                    f = lib.wl_fixed_from_int(arg)
                else:
                    f = lib.wl_fixed_from_double(arg)
                args_ptr[i].f = f
            elif argument.argument_type == ArgumentType.FileDescriptor:
                args_ptr[i].h = arg
            elif argument.argument_type == ArgumentType.String:
                if arg is None:
                    if not argument.nullable:
                        raise Exception
                    new_arg = ffi.NULL
                else:
                    new_arg = ffi.new("char []", arg.encode())
                    refs.append(new_arg)
                args_ptr[i].s = new_arg
            elif argument.argument_type == ArgumentType.Object:
                if arg is None:
                    if not argument.nullable:
                        raise Exception
                    new_arg = ffi.NULL
                else:
                    new_arg = ffi.cast("struct wl_object *", arg._ptr)
                    refs.append(new_arg)
                args_ptr[i].o = new_arg
            elif argument.argument_type == ArgumentType.Array:
                # TODO: this is a bit messy, we probably don't want to put everything in one buffer like this
                new_arg = ffi.new("struct wl_array *")
                new_data = ffi.new("void []", len(arg))
                new_arg.alloc = new_arg.size = len(arg)
                ffi.buffer(new_data)[:] = arg
                refs.append(new_arg)
                refs.append(new_data)

        if len(refs) > 0:
            weakkeydict[args_ptr] = tuple(refs)

        return args_ptr



# This file has been autogenerated by the pywayland scanner

# Copyright © 2013-2016 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .wp_viewport import WpViewport


class WpViewporter(Interface):
    """Surface cropping and scaling

    The global interface exposing surface cropping and scaling capabilities is
    used to instantiate an interface extension for a
    :class:`~pywayland.protocol.wayland.WlSurface` object. This extended
    interface will then allow cropping and scaling the surface contents,
    effectively disconnecting the direct relationship between the buffer and
    the surface size.
    """

    name = "wp_viewporter"
    version = 1

    class error(enum.IntEnum):
        viewport_exists = 0


class WpViewporterProxy(Proxy):
    interface = WpViewporter

    @WpViewporter.request()
    def destroy(self):
        """Unbind from the cropping and scaling interface

        Informs the server that the client will not be using this protocol
        object anymore. This does not affect any other objects,
        :class:`~pywayland.protocol.viewporter.WpViewport` objects included.
        """
        self._marshal(0)
        self._destroy()

    @WpViewporter.request(
        Argument(ArgumentType.NewId, interface=WpViewport),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_viewport(self, surface):
        """Extend surface interface for crop and scale

        Instantiate an interface extension for the given
        :class:`~pywayland.protocol.wayland.WlSurface` to crop and scale its
        content. If the given :class:`~pywayland.protocol.wayland.WlSurface`
        already has a :class:`~pywayland.protocol.viewporter.WpViewport` object
        associated, the viewport_exists protocol error is raised.

        :param surface:
            the surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.viewporter.WpViewport` -- the new
            viewport interface id
        """
        id = self._marshal_constructor(1, WpViewport, surface)
        return id


class WpViewporterResource(Resource):
    interface = WpViewporter


class WpViewporterGlobal(Global):
    interface = WpViewporter


WpViewporter._gen_c()
WpViewporter.proxy_class = WpViewporterProxy
WpViewporter.resource_class = WpViewporterResource
WpViewporter.global_class = WpViewporterGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.client import Display as ClientDisplay
from pywayland.server import Display as ServerDisplay

from pywayland.protocol.wayland import WlCompositor

import threading
import time

got_compositor = None


def _get_registry_callback(proxy, id, iface_name, version):
    global got_compositor
    if iface_name == 'wl_compositor':
        got_compositor = proxy
    return 1


def _run_client():
    global got_compositor
    c = ClientDisplay()

    start = time.time()
    while time.time() < start + 10:
        try:
            c.connect()
        except Exception:
            time.sleep(0.1)
            continue
        break

    reg = c.get_registry()
    reg.dispatcher['global'] = _get_registry_callback

    c.roundtrip()

    c.disconnect()


def _kill_server(data):
    # the `data` is the server Display
    data.terminate()


def test_get_registry():
    global got_compositor
    got_compositor = None

    # run the client in a thread
    client = threading.Thread(target=_run_client)
    client.start()

    # create the server
    s = ServerDisplay()

    # Add a compositor so we can query for it (and keep it alive)
    compositor = WlCompositor.global_class(s)  # noqa

    # Add a timer to kill the server after 0.5 sec (should be more than enough
    # time, don't know a more deterministic way...)
    e = s.get_event_loop()
    source = e.add_timer(_kill_server, data=s)
    source.timer_update(500)

    # start up the server
    s.add_socket()
    s.run()
    s.destroy()

    # wait for the client (shouldn't block on roundtrip once the server is down)
    client.join()

    # make sure we got the compositor in the client
    assert got_compositor



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlDataSource(Interface):
    """Offer to transfer data

    The :class:`WlDataSource` object is the source side of a
    :class:`~pywayland.protocol.wayland.WlDataOffer`. It is created by the
    source client in a data transfer and provides a way to describe the offered
    data and a way to respond to requests to transfer the data.
    """

    name = "wl_data_source"
    version = 3

    class error(enum.IntEnum):
        invalid_action_mask = 0
        invalid_source = 1


class WlDataSourceProxy(Proxy):
    interface = WlDataSource

    @WlDataSource.request(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type):
        """Add an offered mime type

        This request adds a mime type to the set of mime types advertised to
        targets.  Can be called several times to offer multiple types.

        :param mime_type:
            mime type offered by the data source
        :type mime_type:
            `ArgumentType.String`
        """
        self._marshal(0, mime_type)

    @WlDataSource.request()
    def destroy(self):
        """Destroy the data source

        Destroy the data source.
        """
        self._marshal(1)
        self._destroy()

    @WlDataSource.request(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def set_actions(self, dnd_actions):
        """Set the available drag-and-drop actions

        Sets the actions that the source side client supports for this
        operation. This request may trigger :func:`WlDataSource.action()` and
        :func:`WlDataOffer.action()
        <pywayland.protocol.wayland.WlDataOffer.action>` events if the
        compositor needs to change the selected action.

        The dnd_actions argument must contain only values expressed in the
        :func:`WlDataDeviceManager.dnd_actions()
        <pywayland.protocol.wayland.WlDataDeviceManager.dnd_actions>` enum,
        otherwise it will result in a protocol error.

        This request must be made once only, and can only be made on sources
        used in drag-and-drop, so it must be performed before
        :func:`WlDataDevice.start_drag()
        <pywayland.protocol.wayland.WlDataDevice.start_drag>`. Attempting to
        use the source other than for drag-and-drop will raise a protocol
        error.

        :param dnd_actions:
            actions supported by the data source
        :type dnd_actions:
            `ArgumentType.Uint`
        """
        self._marshal(2, dnd_actions)


class WlDataSourceResource(Resource):
    interface = WlDataSource

    @WlDataSource.event(
        Argument(ArgumentType.String, nullable=True),
    )
    def target(self, mime_type):
        """A target accepts an offered mime type

        Sent when a target accepts pointer_focus or motion events.  If a target
        does not accept any of the offered types, type is NULL.

        Used for feedback during drag-and-drop.

        :param mime_type:
            mime type accepted by the target
        :type mime_type:
            `ArgumentType.String` or `None`
        """
        self._post_event(0, mime_type)

    @WlDataSource.event(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def send(self, mime_type, fd):
        """Send the data

        Request for data from the client.  Send the data as the specified mime
        type over the passed file descriptor, then close it.

        :param mime_type:
            mime type for the data
        :type mime_type:
            `ArgumentType.String`
        :param fd:
            file descriptor for the data
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(1, mime_type, fd)

    @WlDataSource.event()
    def cancelled(self):
        """Selection was cancelled

        This data source is no longer valid. There are several reasons why this
        could happen:

        - The data source has been replaced by another data source.

        - The drag-and-drop operation was performed, but the drop destination
          did not accept any of the mime types offered through
          :func:`WlDataSource.target()`.

        - The drag-and-drop operation was performed, but the drop destination
          did not select any of the actions present in the mask offered through
          :func:`WlDataSource.action()`.

        - The drag-and-drop operation was performed but didn't happen over a
          surface.

        - The compositor cancelled the drag-and-drop operation (e.g. compositor
          dependent timeouts to avoid stale drag-and-drop transfers).

        The client should clean up and destroy this data source.

        For objects of version 2 or older, :func:`WlDataSource.cancelled()`
        will only be emitted if the data source was replaced by another data
        source.
        """
        self._post_event(2)

    @WlDataSource.event(version=3)
    def dnd_drop_performed(self):
        """The drag-and-drop operation physically finished

        The user performed the drop action. This event does not indicate
        acceptance, :func:`WlDataSource.cancelled()` may still be emitted
        afterwards if the drop destination does not accept any mime type.

        However, this event might however not be received if the compositor
        cancelled the drag-and-drop operation before this event could happen.

        Note that the data_source may still be used in the future and should
        not be destroyed here.
        """
        self._post_event(3)

    @WlDataSource.event(version=3)
    def dnd_finished(self):
        """The drag-and-drop operation concluded

        The drop destination finished interoperating with this data source, so
        the client is now free to destroy this data source and free all
        associated data.

        If the action used to perform the operation was "move", the source can
        now delete the transferred data.
        """
        self._post_event(4)

    @WlDataSource.event(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def action(self, dnd_action):
        """Notify the selected action

        This event indicates the action selected by the compositor after
        matching the source/destination side actions. Only one action (or none)
        will be offered here.

        This event can be emitted multiple times during the drag-and-drop
        operation, mainly in response to destination side changes through
        :func:`WlDataOffer.set_actions()
        <pywayland.protocol.wayland.WlDataOffer.set_actions>`, and as the data
        device enters/leaves surfaces.

        It is only possible to receive this event after
        :func:`WlDataSource.dnd_drop_performed()` if the drag-and-drop
        operation ended in an "ask" action, in which case the final
        :func:`WlDataSource.action()` event will happen immediately before
        :func:`WlDataSource.dnd_finished()`.

        Compositors may also change the selected action on the fly, mainly in
        response to keyboard modifier changes during the drag-and-drop
        operation.

        The most recent action received is always the valid one. The chosen
        action may change alongside negotiation (e.g. an "ask" action can turn
        into a "move" operation), so the effects of the final action must
        always be applied in :func:`WlDataOffer.dnd_finished()
        <pywayland.protocol.wayland.WlDataOffer.dnd_finished>`.

        Clients can trigger cursor surface changes from this point, so they
        reflect the current action.

        :param dnd_action:
            action selected by the compositor
        :type dnd_action:
            `ArgumentType.Uint`
        """
        self._post_event(5, dnd_action)


class WlDataSourceGlobal(Global):
    interface = WlDataSource


WlDataSource._gen_c()
WlDataSource.proxy_class = WlDataSourceProxy
WlDataSource.resource_class = WlDataSourceResource
WlDataSource.global_class = WlDataSourceGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014, 2015 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlBuffer


class ZwpLinuxBufferParamsV1(Interface):
    """Parameters for creating a dmabuf-based :class:`~pywayland.protocol.wayland.WlBuffer`

    This temporary object is a collection of dmabufs and other parameters that
    together form a single logical buffer. The temporary object may eventually
    create one :class:`~pywayland.protocol.wayland.WlBuffer` unless cancelled
    by destroying it before requesting 'create'.

    Single-planar formats only require one dmabuf, however multi-planar formats
    may require more than one dmabuf. For all formats, an 'add' request must be
    called once per plane (even if the underlying dmabuf fd is identical).

    You must use consecutive plane indices ('plane_idx' argument for 'add')
    from zero to the number of planes used by the drm_fourcc format code. All
    planes required by the format must be given exactly once, but can be given
    in any order. Each plane index can be set only once.
    """

    name = "zwp_linux_buffer_params_v1"
    version = 3

    class error(enum.IntEnum):
        already_used = 0
        plane_idx = 1
        plane_set = 2
        incomplete = 3
        invalid_format = 4
        invalid_dimensions = 5
        out_of_bounds = 6
        invalid_wl_buffer = 7

    class flags(enum.IntFlag):
        y_invert = 1
        interlaced = 2
        bottom_first = 4


class ZwpLinuxBufferParamsV1Proxy(Proxy):
    interface = ZwpLinuxBufferParamsV1

    @ZwpLinuxBufferParamsV1.request()
    def destroy(self):
        """Delete this object, used or not

        Cleans up the temporary data sent to the server for dmabuf-based
        :class:`~pywayland.protocol.wayland.WlBuffer` creation.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def add(self, fd, plane_idx, offset, stride, modifier_hi, modifier_lo):
        """Add a dmabuf to the temporary set

        This request adds one dmabuf to the set in this
        :class:`ZwpLinuxBufferParamsV1`.

        The 64-bit unsigned value combined from modifier_hi and modifier_lo is
        the dmabuf layout modifier. DRM AddFB2 ioctl calls this the fb
        modifier, which is defined in drm_mode.h of Linux UAPI. This is an
        opaque token. Drivers use this token to express tiling, compression,
        etc. driver-specific modifications to the base format defined by the
        DRM fourcc code.

        Warning: It should be an error if the format/modifier pair was not
        advertised with the modifier event. This is not enforced yet because
        some implementations always accept DRM_FORMAT_MOD_INVALID. Also version
        2 of this protocol does not have the modifier event.

        This request raises the PLANE_IDX error if plane_idx is too large. The
        error PLANE_SET is raised if attempting to set a plane that was already
        set.

        :param fd:
            dmabuf fd
        :type fd:
            `ArgumentType.FileDescriptor`
        :param plane_idx:
            plane index
        :type plane_idx:
            `ArgumentType.Uint`
        :param offset:
            offset in bytes
        :type offset:
            `ArgumentType.Uint`
        :param stride:
            stride in bytes
        :type stride:
            `ArgumentType.Uint`
        :param modifier_hi:
            high 32 bits of layout modifier
        :type modifier_hi:
            `ArgumentType.Uint`
        :param modifier_lo:
            low 32 bits of layout modifier
        :type modifier_lo:
            `ArgumentType.Uint`
        """
        self._marshal(1, fd, plane_idx, offset, stride, modifier_hi, modifier_lo)

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def create(self, width, height, format, flags):
        """Create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given dmabufs

        This asks for creation of a
        :class:`~pywayland.protocol.wayland.WlBuffer` from the added dmabuf
        buffers. The :class:`~pywayland.protocol.wayland.WlBuffer` is not
        created immediately but returned via the 'created' event if the dmabuf
        sharing succeeds. The sharing may fail at runtime for reasons a client
        cannot predict, in which case the 'failed' event is triggered.

        The 'format' argument is a DRM_FORMAT code, as defined by the libdrm's
        drm_fourcc.h. The Linux kernel's DRM sub-system is the authoritative
        source on how the format codes should work.

        The 'flags' is a bitfield of the flags defined in enum "flags".
        'y_invert' means the that the image needs to be y-flipped.

        Flag 'interlaced' means that the frame in the buffer is not progressive
        as usual, but interlaced. An interlaced buffer as supported here must
        always contain both top and bottom fields. The top field always begins
        on the first pixel row. The temporal ordering between the two fields is
        top field first, unless 'bottom_first' is specified. It is undefined
        whether 'bottom_first' is ignored if 'interlaced' is not set.

        This protocol does not convey any information about field rate,
        duration, or timing, other than the relative ordering between the two
        fields in one buffer. A compositor may have to estimate the intended
        field rate from the incoming buffer rate. It is undefined whether the
        time of receiving :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` with a new buffer
        attached, applying the :class:`~pywayland.protocol.wayland.WlSurface`
        state, :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` callback trigger,
        presentation, or any other point in the compositor cycle is used to
        measure the frame or field times. There is no support for detecting
        missed or late frames/fields/buffers either, and there is no support
        whatsoever for cooperating with interlaced compositor output.

        The composited image quality resulting from the use of interlaced
        buffers is explicitly undefined. A compositor may use elaborate
        hardware features or software to deinterlace and create progressive
        output frames from a sequence of interlaced input buffers, or it may
        produce substandard image quality. However, compositors that cannot
        guarantee reasonable image quality in all cases are recommended to just
        reject all interlaced buffers.

        Any argument errors, including non-positive width or height, mismatch
        between the number of planes and the format, bad format, bad offset or
        stride, may be indicated by fatal protocol errors: INCOMPLETE,
        INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS.

        Dmabuf import errors in the server that are not obvious client bugs are
        returned via the 'failed' event as non-fatal. This allows attempting
        dmabuf sharing and falling back in the client if it fails.

        This request can be sent only once in the object's lifetime, after
        which the only legal request is destroy. This object should be
        destroyed after issuing a 'create' request. Attempting to use this
        object after issuing 'create' raises ALREADY_USED protocol error.

        It is not mandatory to issue 'create'. If a client wants to cancel the
        buffer creation, it can just destroy this object.

        :param width:
            base plane width in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            base plane height in pixels
        :type height:
            `ArgumentType.Int`
        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param flags:
            see enum flags
        :type flags:
            `ArgumentType.Uint`
        """
        self._marshal(2, width, height, format, flags)

    @ZwpLinuxBufferParamsV1.request(
        Argument(ArgumentType.NewId, interface=WlBuffer),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=2,
    )
    def create_immed(self, width, height, format, flags):
        """Immediately create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given                      dmabufs

        This asks for immediate creation of a
        :class:`~pywayland.protocol.wayland.WlBuffer` by importing the added
        dmabufs.

        In case of import success, no event is sent from the server, and the
        :class:`~pywayland.protocol.wayland.WlBuffer` is ready to be used by
        the client.

        Upon import failure, either of the following may happen, as seen fit by
        the implementation: - the client is terminated with one of the
        following fatal protocol   errors:   - INCOMPLETE, INVALID_FORMAT,
        INVALID_DIMENSIONS, OUT_OF_BOUNDS,     in case of argument errors such
        as mismatch between the number     of planes and the format, bad
        format, non-positive width or     height, or bad offset or stride.   -
        INVALID_WL_BUFFER, in case the cause for failure is unknown or
        plaform specific. - the server creates an invalid
        :class:`~pywayland.protocol.wayland.WlBuffer`, marks it as failed and
        sends a 'failed' event to the client. The result of using this
        invalid :class:`~pywayland.protocol.wayland.WlBuffer` as an argument in
        any request by the client is   defined by the compositor
        implementation.

        This takes the same arguments as a 'create' request, and obeys the same
        restrictions.

        :param width:
            base plane width in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            base plane height in pixels
        :type height:
            `ArgumentType.Int`
        :param format:
            DRM_FORMAT code
        :type format:
            `ArgumentType.Uint`
        :param flags:
            see enum flags
        :type flags:
            `ArgumentType.Uint`
        :returns:
            :class:`~pywayland.protocol.wayland.WlBuffer` -- id for the newly
            created :class:`~pywayland.protocol.wayland.WlBuffer`
        """
        buffer_id = self._marshal_constructor(3, WlBuffer, width, height, format, flags)
        return buffer_id


class ZwpLinuxBufferParamsV1Resource(Resource):
    interface = ZwpLinuxBufferParamsV1

    @ZwpLinuxBufferParamsV1.event(
        Argument(ArgumentType.NewId, interface=WlBuffer),
    )
    def created(self, buffer):
        """Buffer creation succeeded

        This event indicates that the attempted buffer creation was successful.
        It provides the new :class:`~pywayland.protocol.wayland.WlBuffer`
        referencing the dmabuf(s).

        Upon receiving this event, the client should destroy the
        zlinux_dmabuf_params object.

        :param buffer:
            the newly created :class:`~pywayland.protocol.wayland.WlBuffer`
        :type buffer:
            :class:`~pywayland.protocol.wayland.WlBuffer`
        """
        self._post_event(0, buffer)

    @ZwpLinuxBufferParamsV1.event()
    def failed(self):
        """Buffer creation failed

        This event indicates that the attempted buffer creation has failed. It
        usually means that one of the dmabuf constraints has not been
        fulfilled.

        Upon receiving this event, the client should destroy the
        zlinux_buffer_params object.
        """
        self._post_event(1)


class ZwpLinuxBufferParamsV1Global(Global):
    interface = ZwpLinuxBufferParamsV1


ZwpLinuxBufferParamsV1._gen_c()
ZwpLinuxBufferParamsV1.proxy_class = ZwpLinuxBufferParamsV1Proxy
ZwpLinuxBufferParamsV1.resource_class = ZwpLinuxBufferParamsV1Resource
ZwpLinuxBufferParamsV1.global_class = ZwpLinuxBufferParamsV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Callable, Type, Optional
from weakref import WeakKeyDictionary, WeakValueDictionary

from pywayland import ffi
from .argument import Argument
from .message import Message
from .proxy import Proxy
from .resource import Resource
from .globals import Global

weakkeydict: WeakKeyDictionary = WeakKeyDictionary()


class InterfaceMeta(type):
    """Metaclass for Interfaces

    Initializes empty lists for events and requests for the given class.
    """

    def __init__(self, name, bases, dct):
        self.events = []
        self.requests = []

        # Initialize the interface cdata
        self._ptr = ffi.new("struct wl_interface *")


class Interface(metaclass=InterfaceMeta):
    """Wrapper class for wl_wayland structs

    Base class for interfaces that are defined by the wayland.xml class and
    generated by the scanner.  Sub-classes should use the
    :class:`InterfaceMeta` metaclass, which will define subclass.events and
    subclass.requests, the lists of the methods on this interface.  These class
    variables are populated by the :func:`Interface.event` and
    :func:`Interface.request` decorators.
    """

    _ptr: "ffi.InterfaceCData"
    name: str
    version: int
    proxy_class: Type[Proxy]
    resource_class: Type[Resource]
    global_class: Type[Global]

    @classmethod
    def event(cls, *arguments: Argument, version: Optional[int] = None) -> Callable:
        """Decorator for interface events

        Adds the decorated method to the list of events of the interface
        (server-side method).

        :param signature: Encodes the types of the arguments to the decorated
                          function.
        :type signature: `string`
        :param types: List of the types of any objects included in the argument
                      list, None if otherwise.
        :type types: `list`
        """

        def wrapper(func):
            cls.events.append(Message(func, arguments, version))
            return func

        return wrapper

    @classmethod
    def request(cls, *arguments: Argument, version: Optional[int] = None):
        """Decorator for interface requests

        Adds the decorated method to the list of requests of the interface
        (client-side method).

        :param signature: Encodes the types of the arguments to the decorated
                          function.
        :type signature: `string`
        :param types: List of the types of any objects included in the argument
                      list, None if otherwise.
        :type types: list
        """

        def wrapper(func):
            cls.requests.append(Message(func, arguments, version))
            return func

        return wrapper

    @classmethod
    def _gen_c(cls):
        """Creates the wl_interface C struct

        Generates the CFFI cdata for the wl_interface struct given by the
        interface.
        """
        cls.registry = WeakValueDictionary()

        cls._ptr.name = name = ffi.new("char[]", cls.name.encode())
        cls._ptr.version = cls.version

        keep_alive = []
        # Determine the number of methods to assign and assign them
        cls._ptr.method_count = len(cls.requests)
        cls._ptr.methods = methods_ptr = ffi.new(
            "struct wl_message[]", len(cls.requests)
        )
        # Iterate over the methods
        for i, message in enumerate(cls.requests):
            keep_alive.extend(message.build_message_struct(methods_ptr[i]))

        cls._ptr.event_count = len(cls.events)
        cls._ptr.events = events_ptr = ffi.new("struct wl_message[]", len(cls.events))
        # Iterate over the methods
        for i, message in enumerate(cls.events):
            keep_alive.extend(message.build_message_struct(events_ptr[i]))

        weakkeydict[cls._ptr] = (name, methods_ptr, events_ptr) + tuple(keep_alive)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2013-2014 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlOutput


class WpPresentationFeedback(Interface):
    """Presentation time feedback event

    A presentation_feedback object returns an indication that a
    :class:`~pywayland.protocol.wayland.WlSurface` content update has become
    visible to the user. One object corresponds to one content update
    submission (:func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>`). There are two possible
    outcomes: the content update is presented to the user, and a presentation
    timestamp delivered; or, the user did not see the content update because it
    was superseded or its surface destroyed, and the content update is
    discarded.

    Once a presentation_feedback object has delivered a 'presented' or
    'discarded' event it is automatically destroyed.
    """

    name = "wp_presentation_feedback"
    version = 1

    class kind(enum.IntFlag):
        vsync = 0x1
        hw_clock = 0x2
        hw_completion = 0x4
        zero_copy = 0x8


class WpPresentationFeedbackProxy(Proxy):
    interface = WpPresentationFeedback


class WpPresentationFeedbackResource(Resource):
    interface = WpPresentationFeedback

    @WpPresentationFeedback.event(
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def sync_output(self, output):
        """Presentation synchronized to this output

        As presentation can be synchronized to only one output at a time, this
        event tells which output it was. This event is only sent prior to the
        presented event.

        As clients may bind to the same global
        :class:`~pywayland.protocol.wayland.WlOutput` multiple times, this
        event is sent for each bound instance that matches the synchronized
        output. If a client has not bound to the right
        :class:`~pywayland.protocol.wayland.WlOutput` global at all, this event
        is not sent.

        :param output:
            presentation output
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        """
        self._post_event(0, output)

    @WpPresentationFeedback.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def presented(self, tv_sec_hi, tv_sec_lo, tv_nsec, refresh, seq_hi, seq_lo, flags):
        """The content update was displayed

        The associated content update was displayed to the user at the
        indicated time (tv_sec_hi/lo, tv_nsec). For the interpretation of the
        timestamp, see presentation.clock_id event.

        The timestamp corresponds to the time when the content update turned
        into light the first time on the surface's main output. Compositors may
        approximate this from the framebuffer flip completion events from the
        system, and the latency of the physical display path if known.

        This event is preceded by all related sync_output events telling which
        output's refresh cycle the feedback corresponds to, i.e. the main
        output for the surface. Compositors are recommended to choose the
        output containing the largest part of the
        :class:`~pywayland.protocol.wayland.WlSurface`, or keeping the output
        they previously chose. Having a stable presentation output association
        helps clients predict future output refreshes (vblank).

        The 'refresh' argument gives the compositor's prediction of how many
        nanoseconds after tv_sec, tv_nsec the very next output refresh may
        occur. This is to further aid clients in predicting future refreshes,
        i.e., estimating the timestamps targeting the next few vblanks. If such
        prediction cannot usefully be done, the argument is zero.

        If the output does not have a constant refresh rate, explicit video
        mode switches excluded, then the refresh argument must be zero.

        The 64-bit value combined from seq_hi and seq_lo is the value of the
        output's vertical retrace counter when the content update was first
        scanned out to the display. This value must be compatible with the
        definition of MSC in GLX_OML_sync_control specification. Note, that if
        the display path has a non-zero latency, the time instant specified by
        this counter may differ from the timestamp's.

        If the output does not have a concept of vertical retrace or a refresh
        cycle, or the output device is self-refreshing without a way to query
        the refresh count, then the arguments seq_hi and seq_lo must be zero.

        :param tv_sec_hi:
            high 32 bits of the seconds part of the presentation timestamp
        :type tv_sec_hi:
            `ArgumentType.Uint`
        :param tv_sec_lo:
            low 32 bits of the seconds part of the presentation timestamp
        :type tv_sec_lo:
            `ArgumentType.Uint`
        :param tv_nsec:
            nanoseconds part of the presentation timestamp
        :type tv_nsec:
            `ArgumentType.Uint`
        :param refresh:
            nanoseconds till next refresh
        :type refresh:
            `ArgumentType.Uint`
        :param seq_hi:
            high 32 bits of refresh counter
        :type seq_hi:
            `ArgumentType.Uint`
        :param seq_lo:
            low 32 bits of refresh counter
        :type seq_lo:
            `ArgumentType.Uint`
        :param flags:
            combination of 'kind' values
        :type flags:
            `ArgumentType.Uint`
        """
        self._post_event(1, tv_sec_hi, tv_sec_lo, tv_nsec, refresh, seq_hi, seq_lo, flags)

    @WpPresentationFeedback.event()
    def discarded(self):
        """The content update was not displayed

        The content update was never displayed to the user.
        """
        self._post_event(2)


class WpPresentationFeedbackGlobal(Global):
    interface = WpPresentationFeedback


WpPresentationFeedback._gen_c()
WpPresentationFeedback.proxy_class = WpPresentationFeedbackProxy
WpPresentationFeedback.resource_class = WpPresentationFeedbackResource
WpPresentationFeedback.global_class = WpPresentationFeedbackGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_shell_surface import WlShellSurface
from .wl_surface import WlSurface


class WlShell(Interface):
    """Create desktop-style surfaces

    This interface is implemented by servers that provide desktop-style user
    interfaces.

    It allows clients to associate a
    :class:`~pywayland.protocol.wayland.WlShellSurface` with a basic surface.

    Note! This protocol is deprecated and not intended for production use. For
    desktop-style user interfaces, use
    :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell`.
    """

    name = "wl_shell"
    version = 1

    class error(enum.IntEnum):
        role = 0


class WlShellProxy(Proxy):
    interface = WlShell

    @WlShell.request(
        Argument(ArgumentType.NewId, interface=WlShellSurface),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_shell_surface(self, surface):
        """Create a shell surface from a surface

        Create a shell surface for an existing surface. This gives the
        :class:`~pywayland.protocol.wayland.WlSurface` the role of a shell
        surface. If the :class:`~pywayland.protocol.wayland.WlSurface` already
        has another role, it raises a protocol error.

        Only one shell surface can be associated with a given surface.

        :param surface:
            surface to be given the shell surface role
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.wayland.WlShellSurface` -- shell
            surface to create
        """
        id = self._marshal_constructor(0, WlShellSurface, surface)
        return id


class WlShellResource(Resource):
    interface = WlShell


class WlShellGlobal(Global):
    interface = WlShell


WlShell._gen_c()
WlShell.proxy_class = WlShellProxy
WlShell.resource_class = WlShellResource
WlShell.global_class = WlShellGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlKeyboard


class ZwpInputMethodContextV1(Interface):
    """Input method context

    Corresponds to a text input on the input method side. An input method
    context is created on text input activation on the input method side. It
    allows receiving information about the text input from the application via
    events. Input method contexts do not keep state after deactivation and
    should be destroyed after deactivation is handled.

    Text is generally UTF-8 encoded, indices and lengths are in bytes.

    Serials are used to synchronize the state between the text input and an
    input method. New serials are sent by the text input in the commit_state
    request and are used by the input method to indicate the known text input
    state in events like preedit_string, commit_string, and keysym. The text
    input can then ignore events from the input method which are based on an
    outdated state (for example after a reset).

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_input_method_context_v1"
    version = 1


class ZwpInputMethodContextV1Proxy(Proxy):
    interface = ZwpInputMethodContextV1

    @ZwpInputMethodContextV1.request()
    def destroy(self):
        """destroy
        """
        self._marshal(0)
        self._destroy()

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
    )
    def commit_string(self, serial, text):
        """Commit string

        Send the commit string text for insertion to the application.

        The text to commit could be either just a single character after a key
        press or the result of some composing (pre-edit). It could be also an
        empty text when some text should be removed (see
        delete_surrounding_text) or when the input cursor should be moved (see
        cursor_position).

        Any previously set composing text will be removed.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param text:
        :type text:
            `ArgumentType.String`
        """
        self._marshal(1, serial, text)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
        Argument(ArgumentType.String),
    )
    def preedit_string(self, serial, text, commit):
        """Pre-edit string

        Send the pre-edit string text to the application text input.

        The commit text can be used to replace the pre-edit text on reset (for
        example on unfocus).

        Previously sent preedit_style and preedit_cursor requests are also
        processed by the text_input.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param text:
        :type text:
            `ArgumentType.String`
        :param commit:
        :type commit:
            `ArgumentType.String`
        """
        self._marshal(2, serial, text, commit)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def preedit_styling(self, index, length, style):
        """Pre-edit styling

        Set the styling information on composing text. The style is applied for
        length in bytes from index relative to the beginning of the composing
        text (as byte offset). Multiple styles can be applied to a composing
        text.

        This request should be sent before sending a preedit_string request.

        :param index:
        :type index:
            `ArgumentType.Uint`
        :param length:
        :type length:
            `ArgumentType.Uint`
        :param style:
        :type style:
            `ArgumentType.Uint`
        """
        self._marshal(3, index, length, style)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Int),
    )
    def preedit_cursor(self, index):
        """Pre-edit cursor

        Set the cursor position inside the composing text (as byte offset)
        relative to the start of the composing text.

        When index is negative no cursor should be displayed.

        This request should be sent before sending a preedit_string request.

        :param index:
        :type index:
            `ArgumentType.Int`
        """
        self._marshal(4, index)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
    )
    def delete_surrounding_text(self, index, length):
        """Delete text

        Remove the surrounding text.

        This request will be handled on the text_input side directly following
        a commit_string request.

        :param index:
        :type index:
            `ArgumentType.Int`
        :param length:
        :type length:
            `ArgumentType.Uint`
        """
        self._marshal(5, index, length)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def cursor_position(self, index, anchor):
        """Set cursor to a new position

        Set the cursor and anchor to a new position. Index is the new cursor
        position in bytes (when >= 0 this is relative to the end of the
        inserted text, otherwise it is relative to the beginning of the
        inserted text). Anchor is the new anchor position in bytes (when >= 0
        this is relative to the end of the inserted text, otherwise it is
        relative to the beginning of the inserted text). When there should be
        no selected text, anchor should be the same as index.

        This request will be handled on the text_input side directly following
        a commit_string request.

        :param index:
        :type index:
            `ArgumentType.Int`
        :param anchor:
        :type anchor:
            `ArgumentType.Int`
        """
        self._marshal(6, index, anchor)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Array),
    )
    def modifiers_map(self, map):
        """modifiers_map

        :param map:
        :type map:
            `ArgumentType.Array`
        """
        self._marshal(7, map)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def keysym(self, serial, time, sym, state, modifiers):
        """Keysym

        Notify when a key event was sent. Key events should not be used for
        normal text input operations, which should be done with commit_string,
        delete_surrounding_text, etc. The key event follows the
        :class:`~pywayland.protocol.wayland.WlKeyboard` key event convention.
        Sym is an XKB keysym, state is a
        :class:`~pywayland.protocol.wayland.WlKeyboard` key_state.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param time:
        :type time:
            `ArgumentType.Uint`
        :param sym:
        :type sym:
            `ArgumentType.Uint`
        :param state:
        :type state:
            `ArgumentType.Uint`
        :param modifiers:
        :type modifiers:
            `ArgumentType.Uint`
        """
        self._marshal(8, serial, time, sym, state, modifiers)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.NewId, interface=WlKeyboard),
    )
    def grab_keyboard(self):
        """Grab hardware keyboard

        Allow an input method to receive hardware keyboard input and process
        key events to generate text events (with pre-edit) over the wire. This
        allows input methods which compose multiple key events for inputting
        text like it is done for CJK languages.

        :returns:
            :class:`~pywayland.protocol.wayland.WlKeyboard`
        """
        keyboard = self._marshal_constructor(9, WlKeyboard)
        return keyboard

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def key(self, serial, time, key, state):
        """Forward key event

        Forward a :class:`~pywayland.protocol.wayland.WlKeyboard`::key event to
        the client that was not processed by the input method itself. Should be
        used when filtering key events with grab_keyboard.  The arguments
        should be the ones from the
        :class:`~pywayland.protocol.wayland.WlKeyboard`::key event.

        For generating custom key events use the keysym request instead.

        :param serial:
            serial from :class:`~pywayland.protocol.wayland.WlKeyboard`::key
        :type serial:
            `ArgumentType.Uint`
        :param time:
            time from :class:`~pywayland.protocol.wayland.WlKeyboard`::key
        :type time:
            `ArgumentType.Uint`
        :param key:
            key from :class:`~pywayland.protocol.wayland.WlKeyboard`::key
        :type key:
            `ArgumentType.Uint`
        :param state:
            state from :class:`~pywayland.protocol.wayland.WlKeyboard`::key
        :type state:
            `ArgumentType.Uint`
        """
        self._marshal(10, serial, time, key, state)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def modifiers(self, serial, mods_depressed, mods_latched, mods_locked, group):
        """Forward modifiers event

        Forward a :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers
        event to the client that was not processed by the input method itself.
        Should be used when filtering key events with grab_keyboard. The
        arguments should be the ones from the
        :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers event.

        :param serial:
            serial from
            :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers
        :type serial:
            `ArgumentType.Uint`
        :param mods_depressed:
            mods_depressed from
            :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers
        :type mods_depressed:
            `ArgumentType.Uint`
        :param mods_latched:
            mods_latched from
            :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers
        :type mods_latched:
            `ArgumentType.Uint`
        :param mods_locked:
            mods_locked from
            :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers
        :type mods_locked:
            `ArgumentType.Uint`
        :param group:
            group from
            :class:`~pywayland.protocol.wayland.WlKeyboard`::modifiers
        :type group:
            `ArgumentType.Uint`
        """
        self._marshal(11, serial, mods_depressed, mods_latched, mods_locked, group)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
    )
    def language(self, serial, language):
        """language

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param language:
        :type language:
            `ArgumentType.String`
        """
        self._marshal(12, serial, language)

    @ZwpInputMethodContextV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def text_direction(self, serial, direction):
        """text_direction

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param direction:
        :type direction:
            `ArgumentType.Uint`
        """
        self._marshal(13, serial, direction)


class ZwpInputMethodContextV1Resource(Resource):
    interface = ZwpInputMethodContextV1

    @ZwpInputMethodContextV1.event(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def surrounding_text(self, text, cursor, anchor):
        """Surrounding text event

        The plain surrounding text around the input position. Cursor is the
        position in bytes within the surrounding text relative to the beginning
        of the text. Anchor is the position in bytes of the selection anchor
        within the surrounding text relative to the beginning of the text. If
        there is no selected text then anchor is the same as cursor.

        :param text:
        :type text:
            `ArgumentType.String`
        :param cursor:
        :type cursor:
            `ArgumentType.Uint`
        :param anchor:
        :type anchor:
            `ArgumentType.Uint`
        """
        self._post_event(0, text, cursor, anchor)

    @ZwpInputMethodContextV1.event()
    def reset(self):
        """reset
        """
        self._post_event(1)

    @ZwpInputMethodContextV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def content_type(self, hint, purpose):
        """content_type

        :param hint:
        :type hint:
            `ArgumentType.Uint`
        :param purpose:
        :type purpose:
            `ArgumentType.Uint`
        """
        self._post_event(2, hint, purpose)

    @ZwpInputMethodContextV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def invoke_action(self, button, index):
        """invoke_action

        :param button:
        :type button:
            `ArgumentType.Uint`
        :param index:
        :type index:
            `ArgumentType.Uint`
        """
        self._post_event(3, button, index)

    @ZwpInputMethodContextV1.event(
        Argument(ArgumentType.Uint),
    )
    def commit_state(self, serial):
        """commit_state

        :param serial:
            serial of text input state
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(4, serial)

    @ZwpInputMethodContextV1.event(
        Argument(ArgumentType.String),
    )
    def preferred_language(self, language):
        """preferred_language

        :param language:
        :type language:
            `ArgumentType.String`
        """
        self._post_event(5, language)


class ZwpInputMethodContextV1Global(Global):
    interface = ZwpInputMethodContextV1


ZwpInputMethodContextV1._gen_c()
ZwpInputMethodContextV1.proxy_class = ZwpInputMethodContextV1Proxy
ZwpInputMethodContextV1.resource_class = ZwpInputMethodContextV1Resource
ZwpInputMethodContextV1.global_class = ZwpInputMethodContextV1Global
